if UnitLevel("player") >= 110 then
	return;
end

local LSM = LibStub("LibSharedMedia-3.0");
local font = LSM:Fetch("font", "MeatEdition");
local tex = LSM:Fetch("statusbar", "HalO");

local E, L = unpack(select(2, ...));
local F = E.Func;

local Experience = E:NewModule("Experience", "AceEvent-3.0");

do
	local Indicator = CreateFrame("Frame", nil, UIParent);
	Indicator:SetMovable(true);
	Indicator:EnableMouse(true);
	Indicator:RegisterForDrag("LeftButton");
	Indicator:SetScript("OnDragStart", Indicator.StartMoving);
	Indicator:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing();

		Experience:UpdateDB();
	end);
	Indicator:SetSize(280, 50);
	F.SetTemplate(Indicator, true);
	local r, g, b = Indicator.template:GetBackdropColor();
	Indicator.template:SetBackdropColor(r, g, b, 0.7);

	Experience.Indicator = Indicator;

	local XPBar = CreateFrame("StatusBar", nil, Indicator);
	XPBar:SetStatusBarTexture(tex);
	XPBar:SetMinMaxValues(0, 100);
	XPBar:SetSize(Indicator:GetWidth(), 5);
	XPBar:SetPoint("TOP", Indicator, "BOTTOM", 0, -3);
	F.SetTemplate(XPBar, true);
	XPBar.template:SetBackdropColor(0, 0, 0, 0);
	F.SetBackground(XPBar);

	Experience.XPBar = XPBar;

	XPBar.Line = {};
	for i = 1, 9 do
		local Line = XPBar:CreateTexture(nil, "OVERLAY");
		Line:SetColorTexture(0, 0, 0);
		Line:SetSize(1, XPBar:GetHeight());
		Line:SetPoint("LEFT", i * XPBar:GetWidth() / 10, 0);

		XPBar.Line[i] = Line;
	end

	local _, class = UnitClass("player");
	local color = RAID_CLASS_COLORS[class];

	local NameText = Indicator:CreateFontString(nil, "OVERLAY");
	NameText:SetFont(font, 16, "OUTLINE");
	NameText:SetPoint("TOP", 0, -5);
	NameText:SetTextColor(color.r, color.g, color.b);
	NameText:SetText("");

	Indicator.NameText = NameText;

	local XPText = Indicator:CreateFontString(nil, "OVERLAY");
	XPText:SetFont(font, 14, "OUTLINE");
	XPText:SetPoint("BOTTOM", 0, 5);
	XPText:SetTextColor(1, 1, 1);
	XPText:SetFormattedText("|cffeda900Level:|r %d, |cffeda900XP:|r %s / %s (%.2f%%)", 0, 0, 0, 0);

	Indicator.XPText = XPText;
end

function Experience:CalculateXP()
	local curXP, maxXP = UnitXP("player"), UnitXPMax("player");
	local perXP = curXP / maxXP * 100;

	return curXP, maxXP, perXP;
end

function Experience:UpdateText()
	local level = UnitLevel("player");
	local curXP, maxXP, perXP = self:CalculateXP();

	self.Indicator.NameText:SetText(UnitName("player"));
	self.Indicator.XPText:SetFormattedText("|cffeda900Level:|r %d, |cffeda900XP:|r %s / %s (%.2f%%)", level, F.CommaValue(curXP), F.CommaValue(maxXP), perXP);
end

function Experience:UpdateBar()
	local curXP, maxXP, perXP = self:CalculateXP();

	local id = GetRestState();

	if id == 1 then
		self.XPBar:SetStatusBarColor(0.2, 0.6, 1);
		self.XPBar.background:SetVertexColor(0.2 * 0.25, 0.6 * 0.25, 1 * 0.25, 0.7);
	else
		self.XPBar:SetStatusBarColor(1, 0.2, 0.8);
		self.XPBar.background:SetVertexColor(1 * 0.25, 0.2 * 0.25, 0.8 * 0.25, 0.7);
	end

	self.XPBar:SetValue(perXP);
end

function Experience:UpdateDB()
	local point, relativeFrame, relativePoint, ofsx, ofsy = self.Indicator:GetPoint();

	if relativeFrame == UIParent then
		relativeFrame = "UIParent";
	end

	E.db.global = {
		experience = {
			["point"] = point,
			["relativeFrame"] = relativeFrame,
			["relativePoint"] = relativePoint,
			["ofsx"] = ofsx,
			["ofsy"] = ofsy,
		},
	}
end

function Experience:PLAYER_ENTERING_WORLD()
	local db = E.db;
	local point, relativeFrame, relativePoint, ofsx, ofsy;

	if type(db.global) == "nil" then
		point = db.defaults.global.experience.point;
		relativeFrame = db.defaults.global.experience.relativeFrame;
		relativePoint = db.defaults.global.experience.relativePoint;
		ofsx = db.defaults.global.experience.ofsx;
		ofsy = db.defaults.global.experience.ofsy;
	else
		point = db.global.experience.point;
		relativeFrame = db.global.experience.relativeFrame;
		relativePoint = db.global.experience.relativePoint;
		ofsx = db.global.experience.ofsx;
		ofsy = db.global.experience.ofsy;
	end

	self.Indicator:SetPoint(point, relativeFrame, relativePoint, ofsx, ofsy);

	self:UpdateText();
	self:UpdateBar();
end

function Experience:PLAYER_XP_UPDATE()
	self:UpdateText();
	self:UpdateBar();
end

function Experience:PLAYER_UPDATE_RESTING()
	self:UpdateBar();
end

function Experience:PLAYER_LEVEL_UP(level, hp, mp, talentPoints, strength, agility, stamina, intellect, spirit)
	if level == 110 then
		self:UnregisterEvent("PLAYER_LOGIN");
		self:UnregisterEvent("PLAYER_XP_UPDATE");
		self:UnregisterEvent("PLAYER_UPDATE_RESTING");
		self:UnregisterEvent("PLAYER_LEVEL_UP");
		self:UnregisterEvent("PLAYER_LOGOUT");

		self:UpdateDB();

		self.XPBar.background = nil;
		self.XPBar.template = nil;
		for i = 1, #(self.XPBar.Line) do
			self.XPBar.Line[i] = nil;
		end
		self.XPBar = nil;

		self.Indicator.XPText = nil;
		self.Indicator.NameText = nil;

		self.Indicator.template = nil;
		self.Indicator = nil;
	end
end

function Experience:PLAYER_LOGOUT()
	self:UpdateDB();
end

Experience:RegisterEvent("PLAYER_ENTERING_WORLD");
Experience:RegisterEvent("PLAYER_XP_UPDATE");
Experience:RegisterEvent("PLAYER_UPDATE_RESTING");
Experience:RegisterEvent("PLAYER_LEVEL_UP");
Experience:RegisterEvent("PLAYER_LOGOUT");